package fr.velossity.sample.adapter.device.impl;

import java.util.Hashtable;
import java.util.Properties;

import org.apache.felix.ipojo.ComponentInstance;
import org.apache.felix.ipojo.Factory;

import fr.velossity.sample.adaptee.AdapteeDevice;

/**
 * This factory is responsible to create dynamically a Device adapter when a low
 * device is appearing and to delete it when it is disappearing.
 * 
 * @author C. Saint-Marcel
 * 
 */
public class DeviceAdapterCreator {

	/**
	 * References all the adapters to unregister them when the low device is
	 * disappearing.
	 */
	Hashtable /* <UUID, Device> */highDevices;

	/**
	 * iPOJO injected field used to create iPOJO instances.
	 */
	Factory deviceFactory;

	/**
	 * Constructor for iPOJO.
	 */
	public DeviceAdapterCreator() {
		highDevices = new Hashtable();
	}

	/**
	 * iPOJO callback to bind a new low device.
	 * 
	 * @param dev
	 *            the disappearing device
	 */
	public void bindLowDevice(AdapteeDevice dev) {
		Properties props = new Properties();
		props.put("low.device", dev);
		ComponentInstance instance;
		try {
			// Create the instance
			instance = deviceFactory
					.createComponentInstance(props);
			// Store the reference to the new iPOJO instance
			highDevices.put(dev, instance);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * iPOJO callback to unbind a low device.
	 * 
	 * @param dev the disappearing device
	 */
	public void unbindLowDevice(AdapteeDevice dev) {
		ComponentInstance instance = (ComponentInstance) highDevices
				.remove(dev);
		if (instance != null) {
			instance.dispose();
		}
	}

}
